pub enum Paint {
Solid(PremulColor),
Indexed(IndexedPaint),
}Expand description
A paint used internally by a rendering frontend to store how a draw should be painted. There are only two types of paint:
- Simple solid colors, which are stored in premultiplied representation so that the renderer doesn’t have to recompute it.
- Indexed paints, which can represent any arbitrary, more complex paint that is
determined by the frontend. The intended way of using this is to store a vector
of paints and store its index inside
IndexedPaint.
Variants
Solid(PremulColor)
A premultiplied RGBA8 color.
Indexed(IndexedPaint)
A paint that needs to be resolved via an index.
Implementations
Sourceimpl Paint
impl Paint
Sourcepub fn may_have_transparency(&self, encoded_paints: &[EncodedPaint]) -> bool
pub fn may_have_transparency(&self, encoded_paints: &[EncodedPaint]) -> bool
Returns whether this paint may produce non-opaque pixels.