pub struct Glyph {
pub id: u32,
pub x: f32,
pub y: f32,
}Available on crate feature
text only.Expand description
Positioned glyph.
Fields
id: u32The font-specific identifier for this glyph.
This ID is specific to the font being used and corresponds to the glyph index within that font. It is not a Unicode code point.
x: f32X-offset in run, relative to transform.
y: f32Y-offset in run, relative to transform.